----------------------------------------------------------------------------------------
--   Script:		Camera Manipulators - DOF
--	 Version:		v0.41
--	 Started:       01/31/00
--	 Last Modified: 02/01/00
--   Description:	Adds FocalDepth, FOV, Environment and Clipping Ranges manipulators to the Cameras
--	 Documentation: Visit http://www.gfxcentral.com/bobo/mxs4
--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at
----------------------------------------------------------------------------------------

plugin simpleManipulator CameraDOFManip
name:"CameraDOFManip"
invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected

    on canManipulate target return (superclassof target == Camera and (hasProperty target.multipass_effect #focalDepth))

    on updateGizmos do
    (
	if (hasProperty target.multipass_effect #focalDepth) then
	(
        this.clearGizmos()
        giz = manip.makeCircle [0,0,(-target.multipass_effect.focalDepth )] 20 28

        giz2 = manip.makeGizmoShape()
        giz2.AddPoint [20,0,-target.multipass_effect.focalDepth]
        giz2.AddPoint [-20,0,-target.multipass_effect.focalDepth]

        giz3 = manip.makeGizmoShape()
        giz3.AddPoint [0,20,-target.multipass_effect.focalDepth]
        giz3.AddPoint [0,-20,-target.multipass_effect.focalDepth]
		
        this.addGizmoShape giz 0 greenColor redColor  
        this.addGizmoShape giz2 0 greenColor redColor  
        this.addGizmoShape giz3 0 greenColor redColor  
		
        return node.name + " FocalDepth = " + target.multipass_effect.focalDepth as string 
	)	
    )

    on mouseMove m which do
    (
        local plane1 = manip.makePlaneFromNormal x_axis [0, 0, 0]
        local plane2 = manip.makePlaneFromNormal y_axis [0, 0, 0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        local pl = plane1.mostOrthogonal viewRay plane2
        res = pl.intersect viewRay &projectedPoint
        if res and (hasProperty target.multipass_effect #focalDepth) then target.multipass_effect.focalDepth = length projectedPoint
    )
)



plugin simpleManipulator CameraFOVManip
name:"CameraFOVManip"
invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local t_dist
	
    on canManipulate target return (superclassof target == Camera )

    on updateGizmos do
    (
        this.clearGizmos()
        giz = manip.makeGizmoShape()
		if target.TargetDistance == 0 then
			t_dist = node.TargetDistance
			else
			t_dist = target.TargetDistance
			
        giz.AddPoint [t_dist * tan(target.FOV/2),t_dist * tan(target.FOV/2)/GetRendImageAspect(),(-t_dist)]
        giz.AddPoint [t_dist * tan(target.FOV/2),-t_dist * tan(target.FOV/2)/GetRendImageAspect(),(-t_dist)]
		
        this.addGizmoShape giz 0 greenColor redColor  
        return node.name + " FOV = " + target.FOV as string 
    )

    on mouseMove m which do
    (
        local plane1 = manip.makePlaneFromNormal z_axis [0,0,(-t_dist)]
        local projectedPoint = [0,0,(-t_dist)]
        viewRay = this.getLocalViewRay m
        res = plane1.intersect viewRay &projectedPoint
        if res then target.FOV = 2* atan (length (projectedPoint - [0,0,(-t_dist)]) /(t_dist) )
    )
)


plugin simpleManipulator CameraERNearManip
name:"CameraERNearManip"
invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local er_n 
    on canManipulate target return (superclassof target == Camera and target.showranges)

    on updateGizmos do
    (
        this.clearGizmos()
        giz = manip.makeGizmoShape()
		er_n = target.nearrange
        giz.AddPoint [er_n * tan(target.FOV/2) * 0.7 ,er_n * 0.7 * tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        giz.AddPoint [er_n * tan(target.FOV/2) * 0.7 ,-er_n * 0.7 *tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        giz.AddPoint [-er_n * tan(target.FOV/2)* 0.7 ,er_n * 0.7* tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        giz.AddPoint [-er_n * tan(target.FOV/2)* 0.7 ,-er_n * 0.7* tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
		
        this.addGizmoShape giz 0 greenColor redColor  
        return node.name + " NearRange = " + target.nearrange as string 
    )

    on mouseMove m which do
    (
        local plane1 = manip.makePlaneFromNormal x_axis [0, 0, (-er_n)]
        local plane2 = manip.makePlaneFromNormal y_axis [0, 0, (-er_n)]
        local projectedPoint = [0,0,(-er_n)]
        viewRay = this.getLocalViewRay m
        local pl = plane1.mostOrthogonal viewRay plane2
        res = pl.intersect viewRay &projectedPoint
        if res then target.nearrange = length (projectedPoint)
		if target.nearrange > target.farrange then target.farrange = target.nearrange
    )
)


plugin simpleManipulator CameraERFarManip
name:"CameraERFarManip"
invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local er_n 
    on canManipulate target return (superclassof target == Camera and target.showranges)

    on updateGizmos do
    (
        this.clearGizmos()
		er_n = target.farrange
        giz = manip.makeGizmoShape()
        giz.AddPoint [-er_n * tan(target.FOV/2) * 0.7 ,-er_n * 0.7 * tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        giz.AddPoint [-er_n * tan(target.FOV/2) * 0.7 ,0.0 ,(-er_n)]
        giz.AddPoint [er_n * tan(target.FOV/2) * 0.7 ,0.0 ,(-er_n)]
        giz.AddPoint [-er_n * tan(target.FOV/2) * 0.7 ,0.0 ,(-er_n)]
        giz.AddPoint [-er_n * tan(target.FOV/2)* 0.7 ,er_n * 0.7* tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        giz.AddPoint [er_n * tan(target.FOV/2)* 0.7 ,er_n * 0.7* tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
		
        this.addGizmoShape giz 0 greenColor redColor  
        return node.name + " FarRange = " + target.farrange as string 
    )

    on mouseMove m which do
    (
        local plane1 = manip.makePlaneFromNormal x_axis [0, 0, (-er_n)]
        local plane2 = manip.makePlaneFromNormal y_axis [0, 0, (-er_n)]
        local projectedPoint = [0,0,(-er_n)]
        viewRay = this.getLocalViewRay m
        local pl = plane1.mostOrthogonal viewRay plane2
        res = pl.intersect viewRay &projectedPoint
        if res then target.farrange = length (projectedPoint)
		if target.nearrange > target.farrange then target.nearrange = target.farrange 
		
    )
)



plugin simpleManipulator CameraFarClipManip
name:"CameraFarClipManip"
invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local er_n 
    on canManipulate target return (superclassof target == Camera and target.clipManually)

    on updateGizmos do
    (
        this.clearGizmos()
		er_n = target.farclip
        giz = manip.makeGizmoShape()
        giz.AddPoint [-er_n * tan(target.FOV/2) ,-er_n * tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        giz.AddPoint [er_n * tan(target.FOV/2) ,er_n * tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        this.addGizmoShape giz 0 greenColor redColor  

        giz2 = manip.makeGizmoShape()
        giz2.AddPoint [er_n * tan(target.FOV/2) ,-er_n * tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        giz2.AddPoint [-er_n * tan(target.FOV/2) ,er_n * tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        this.addGizmoShape giz2 0 greenColor redColor  
		
        return node.name + " FarClipping = " + target.farclip as string 
    )

    on mouseMove m which do
    (
        local plane1 = manip.makePlaneFromNormal x_axis [0, 0, (-er_n)]
        local plane2 = manip.makePlaneFromNormal y_axis [0, 0, (-er_n)]
        local projectedPoint = [0,0,(-er_n)]
        viewRay = this.getLocalViewRay m
        local pl = plane1.mostOrthogonal viewRay plane2
        res = pl.intersect viewRay &projectedPoint
        if res then target.farclip = length (projectedPoint)
		if target.nearclip > target.farclip then target.nearclip = target.farclip
    )
)


plugin simpleManipulator CameraNearClipManip
name:"CameraNearClipManip"
invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local er_n 
    on canManipulate target return (superclassof target == Camera and target.clipManually)

    on updateGizmos do
    (
        this.clearGizmos()
		er_n = target.nearclip
        giz = manip.makeGizmoShape()
        giz.AddPoint [-er_n * tan(target.FOV/2) ,-er_n * tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        giz.AddPoint [er_n * tan(target.FOV/2) ,er_n * tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        this.addGizmoShape giz 0 greenColor redColor  

        giz2 = manip.makeGizmoShape()
        giz2.AddPoint [er_n * tan(target.FOV/2) ,-er_n * tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        giz2.AddPoint [-er_n * tan(target.FOV/2) ,er_n * tan(target.FOV/2)/GetRendImageAspect(),(-er_n)]
        this.addGizmoShape giz2 0 greenColor redColor  
		
        return node.name + " NearClipping = " + target.nearclip as string 
    )

    on mouseMove m which do
    (
        local plane1 = manip.makePlaneFromNormal x_axis [0, 0, (-er_n)]
        local plane2 = manip.makePlaneFromNormal y_axis [0, 0, (-er_n)]
        local projectedPoint = [0,0,(-er_n)]
        viewRay = this.getLocalViewRay m
        local pl = plane1.mostOrthogonal viewRay plane2
        res = pl.intersect viewRay &projectedPoint
        if res then target.nearclip = length (projectedPoint)
		if target.nearclip > target.farclip then target.farclip = target.nearclip
		
    )
)