----------------------------------------------------------------------------------------
--   Script:		MAGMApulators: Lights Manipulators 
--	 Version:		v0.42
--	 Started:       11/01/00
--	 Last Modified: 08/28/02
--   Description:	Adds Manipulators for Lights Decay, Near and Far Ranges, 
--                  Direct Lights Falloff and Hotspot.
--	 Documentation: Visit http://www.scriptspot.com/bobo/mxs4
--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at
--   Based on the Radius Manipulator code by Scott Morrison from Discreet.
----------------------------------------------------------------------------------------
--   New in v0.42
--  *Added hasProperty checks to avoid crashes with Skylight in max 5.
--  *Fixed some general code formatting
----------------------------------------------------------------------------------------


persistent global lights_manipulators_enabled
if lights_manipulators_enabled == undefined then 
(
	lights_manipulators_enabled = #()
	for i = 1 to 20 do append lights_manipulators_enabled true 
)


plugin simpleManipulator LightDecayManip
    name:"LightDecayManip"
    invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	
    -- This manipulator affects the Light's Decay Property:
    on canManipulate target return
		(superclassof target == Light) and lights_manipulators_enabled[1] and hasProperty target "decayRadius"
		
    on updateGizmos do
    (
        this.clearGizmos()
		local size = target.decayRadius
		giz2 = manip.MakeCircle [0,0,0] size 24 
		my_tm = matrixFromNormal [0,1,0]
		giz2.transform my_tm
        this.addGizmoShape giz2 0 greenColor redColor  
        this.addGizmoText "Decay" [0,0,-size] GizmoDontHitTest (color 1 1 1) redColor  
        return node.name + " Decay = " + target.decayRadius as string 
    )

    on mouseMove m which do
    (
        local plane = manip.makePlaneFromNormal y_axis [0, 0, 0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = plane.intersect viewRay &projectedPoint
        if (res) then 
			target.decayRadius = sqrt(projectedPoint.x^2+ projectedPoint.z^2)
    )
)


plugin simpleManipulator LightFarStartManip
    name:"LightFarStartManip"
    invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	
    -- This manipulator affects the Light's Far Attenuation Start Property:
    on canManipulate target return
		(superclassof target == Light) and lights_manipulators_enabled[4] and hasProperty target "farAttenStart"
		
    on updateGizmos do
    (
        this.clearGizmos()
		local size = target.farAttenStart
		giz2 = manip.MakeCircle [0,0,0] size 24 
		my_tm = matrixFromNormal [0,1,0]
		giz2.transform my_tm		
        this.addGizmoShape giz2 0 greenColor redColor  
        this.addGizmoText "Far Atten.Start" [size,0,0] GizmoDontHitTest (color 1 1 1) redColor  
        return node.name + " Far Atten.Start= " + target.farAttenStart as string 
    )

    on mouseMove m which do
    (
        local plane = manip.makePlaneFromNormal y_axis [0, 0, 0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = plane.intersect viewRay &projectedPoint
        if res then 
		(
			result = sqrt(projectedPoint.x^2+ projectedPoint.z^2)
			target.farAttenStart = result				
			if result > target.farAttenEnd then target.farAttenEnd = target.farAttenStart
			if result < target.nearAttenEnd then target.nearAttenEnd = target.farAttenStart
			if result < target.nearAttenStart then target.nearAttenStart = target.farAttenStart
		)
    )
)


plugin simpleManipulator LightFarEndManip
    name:"LightFarEndManip"
    invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	
    -- This manipulator affects the Light's Far Attenuation End Property:
    on canManipulate target return
		(superclassof target == Light) and lights_manipulators_enabled[5] and hasProperty target "farAttenEnd"
		
    on updateGizmos do
    (
        this.clearGizmos()
		local size = target.farAttenEnd
		giz2 = manip.MakeCircle [0,0,0] size 24 
		my_tm = matrixFromNormal [0,1,0]
		giz2.transform my_tm		
        this.addGizmoShape giz2 0 greenColor redColor  
        this.addGizmoText "Far Atten.End" [-size,0,0] GizmoDontHitTest (color 1 1 1) redColor  
        return node.name + " Far Atten.End= " + target.farAttenStart as string 
    )

    on mouseMove m which do
    (
        local plane = manip.makePlaneFromNormal y_axis [0, 0, 0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = plane.intersect viewRay &projectedPoint
        if res then
		(
			result = sqrt(projectedPoint.x^2+ projectedPoint.z^2)
			target.farAttenEnd  = result				
			if result < target.farAttenStart then target.farAttenStart = target.farAttenEnd
			if result < target.nearAttenStart then target.nearAttenStart = target.farAttenEnd
			if result < target.nearAttenEnd then target.nearAttenEnd = target.farAttenEnd
		)	
    )
)


plugin simpleManipulator LightNearStartManip
    name:"LightNearStartManip"
    invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	
    -- This manipulator affects the Light's Near Attenuation Start Property:
    on canManipulate target return
		(superclassof target == Light) and lights_manipulators_enabled[2] and hasProperty target "nearAttenStart"

    on updateGizmos do
    (
        this.clearGizmos()
		local size = target.nearAttenStart
		giz2 = manip.MakeCircle [0,0,0] size 24 
		my_tm = matrixFromNormal [1,0,0]
		giz2.transform my_tm
        this.addGizmoShape giz2 0 greenColor redColor  
        this.addGizmoText "Near Atten.Start" [0,size,0] GizmoDontHitTest (color 1 1 1) redColor  
        return node.name + " Near Atten.Start= " + target.nearAttenStart as string 
    )

    on mouseMove m which do
    (
        local plane = manip.makePlaneFromNormal x_axis [0, 0, 0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = plane.intersect viewRay &projectedPoint
        if res then 
		(
			result = sqrt(projectedPoint.y^2+ projectedPoint.z^2)
			target.nearAttenStart = result				
			if result > target.nearAttenEnd then target.nearAttenEnd = target.nearAttenStart
			if result > target.farAttenStart then target.farAttenStart = target.nearAttenStart
			if result > target.farAttenEnd then target.farAttenEnd = target.nearAttenStart
		)
    )
)


plugin simpleManipulator LightNearEndManip
    name:"LightNearEndManip"
    invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	
    -- This manipulator affects the Light's Near Attenuation End Property:
    on canManipulate target return
		(superclassof target == Light) and lights_manipulators_enabled[3] and hasProperty target "nearAttenEnd"

    on updateGizmos do
    (
        this.clearGizmos()
		local size = target.nearAttenEnd
		giz2 = manip.MakeCircle [0,0,0] size 24 
		my_tm = matrixFromNormal [1,0,0]
		giz2.transform my_tm		
        this.addGizmoShape giz2 0 greenColor redColor  
        this.addGizmoText "Near Atten.End" [0,-size,0] GizmoDontHitTest (color 1 1 1) redColor  
        return node.name + " Near Atten.End= " + target.nearAttenStart as string 
    )

    on mouseMove m which do
    (
        local plane = manip.makePlaneFromNormal x_axis [0, 0, 0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = plane.intersect viewRay &projectedPoint
        if res then
		(
			result = sqrt(projectedPoint.y^2+ projectedPoint.z^2)
			target.nearAttenEnd  = result				
			if result < target.nearAttenStart then target.nearAttenStart = target.nearAttenEnd
			if result > target.farAttenStart then target.farAttenStart = target.nearAttenEnd
			if result > target.farAttenEnd then target.farAttenEnd = target.nearAttenEnd
		)	
    )
)



plugin simpleManipulator DirectFalloffManip
name:"DirectFalloffManip"
invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected

    on canManipulate target return (classof target == TargetDirectionallight or classof target == Directionallight) and lights_manipulators_enabled[6] 

    on updateGizmos do
    (
        this.clearGizmos()
        giz = manip.makeCircle [0,0,0] (target.falloff * 1.01) 28
        this.addGizmoShape giz 0 greenColor redColor  
        return node.name + " Falloff = " + target.falloff as string 
    )

    on mouseMove m which do
    (
        local pl = manip.makePlaneFromNormal z_axis [0, 0, 0],
        projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = pl.intersect viewRay &projectedPoint
        if (res) then target.falloff = length projectedPoint
		if target.hotspot > target.falloff then target.hotspot = target.falloff - 1.0 
    )
)


plugin simpleManipulator DirectHotspotManip
name:"DirectHotspotManip"
invisible:true
(
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected

    on canManipulate target return (classof target == TargetDirectionallight or classof target == Directionallight) and lights_manipulators_enabled[6]

    on updateGizmos do
    (
        this.clearGizmos()
        giz = manip.makeCircle [0,0,0] (target.hotspot) 28
        this.addGizmoShape giz 0 greenColor redColor  
        return node.name + " Hotspot = " + target.hotspot as string 
    )

    on mouseMove m which do
    (
        local pl = manip.makePlaneFromNormal z_axis [0, 0, 0],
        projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = pl.intersect viewRay &projectedPoint
        if (res) then target.hotspot = length projectedPoint
		if target.hotspot > target.falloff then target.falloff = target.hotspot+1.0 

    )
)

