----------------------------------------------------------------------------------------
--   Script:		MAGMApulators:Modifier Manipulators 
--	 Version:		v0.4
--	 Started:       10/30/00
--	 Last Modified: 01/29/01
--   Description:	Adds Manipulators for Bend and Twist Angles, Bend and Skew Directions and Skew Amount.
--	 Documentation: Visit http://www.gfxcentral.com/bobo/mxs4
--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at
--   Based on the Radius Manipulator code by Scott Morrison from Discreet.
--   Angle Warpping Code in Angle Manipulator courtesy Scott Morrison. (Thanks!)
----------------------------------------------------------------------------------------


persistent global modifier_manipulators_enabled
if modifier_manipulators_enabled == undefined then 
(
modifier_manipulators_enabled = #()
for i = 1 to 20 do append modifier_manipulators_enabled true 
)


plugin simpleManipulator AngleManip
    name:"AngleManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local a_mode = 0
	local windings = 0
	
    -- This manipulator affects any modifier with an "Angle" property:
	--*Bend Angle
	--*Twist Angle
	
    on canManipulate target return
		(
		(try(if target.angle != undefined then true)catch(false)) and modifier_manipulators_enabled[1] and superclassof target != Shape
		)

    -- Create the manipulator gizmo.
    -- This is called initially and whenever the manipulator target changes
	
    on updateGizmos do
    (
        -- Clear the current gizmo cache
        this.clearGizmos()
	
		-- Create Gizmo 
		
		local size1 = length (node.max - node.min) / 50

	
		a_mode = 2
		case (classof target) of
		(
		Bend: (
				size2 = size1*20
				a_mode = 1
				)
		Twist: (
				size2 = size1*25
				a_mode = 2
				)
		)
		
		giz2 = manip.MakeCircle [0,0,0] size2 24
		
		case a_mode of
		(
		1: (
			giz1 = manip.MakeSphere [size2*cos(-target.angle+90),0,size2*sin(-target.angle+90)] size1 8
			my_tm = matrixFromNormal [0,1,0]
			giz2.transform  my_tm
	        giz3 = manip.makeGizmoShape()
	        giz3.addPoint [size2*cos(-target.angle+90),0,size2*sin(-target.angle+90)]
	        giz3.addPoint [0,0,0] 			
			)
		2: (
			giz1 = manip.MakeSphere [size2*cos(-target.angle+90),size2*sin(-target.angle+90),0] size1 8
	        giz3 = manip.makeGizmoShape()
	        giz3.addPoint [size2*cos(-target.angle+90),size2*sin(-target.angle+90),0]
	        giz3.addPoint [0,0,0] 			
			)
		)
		
        -- Add the Gizmos to the manipulator
		
        this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoShape giz2 gizmoDontHitTest greenColor redColor  
        this.addGizmoShape giz3 gizmoDontHitTest greenColor redColor  
		
        return node.name + ":" +target.name +" Angle = " + target.angle as string 
    )

    on mouseMove m which do
    (
		local old_angle = target.angle 
		local windings = int(old_angle / 360.0)
		if old_angle < 0 then windings -=1
		local norm_angle = old_angle - (windings * 360.0)
		
		case a_mode of
		(
		1: (
	        local plane = manip.makePlaneFromNormal y_axis [0, 0, 0]
	        local projectedPoint = [0,0,0]
	        viewRay = this.getLocalViewRay m
	        res = plane.intersect viewRay &projectedPoint
	        if (res) then 
				(
				if projectedPoint.x > 0 and projectedPoint.z > 0 then result = atan (projectedPoint.x/projectedPoint.z)
				if projectedPoint.x < 0 and projectedPoint.z > 0 then result = atan (projectedPoint.x/projectedPoint.z)
				if projectedPoint.x > 0 and projectedPoint.z < 0 then result = (atan (projectedPoint.x/projectedPoint.z) + 180)
				if projectedPoint.x < 0 and projectedPoint.z < 0 then result = (atan (projectedPoint.x/projectedPoint.z) - 180)
				)
			)
		2: (
    	    local plane = manip.makePlaneFromNormal z_axis [0, 0, 0]
	        local projectedPoint = [0,0,0]
	        viewRay = this.getLocalViewRay m
	        res = plane.intersect viewRay &projectedPoint
	        if (res) then 
				(
				if projectedPoint.x > 0 and projectedPoint.y > 0 then result = atan (projectedPoint.x/projectedPoint.y)
				if projectedPoint.x < 0 and projectedPoint.y > 0 then result = atan (projectedPoint.x/projectedPoint.y)
				if projectedPoint.x > 0 and projectedPoint.y < 0 then result = (atan (projectedPoint.x/projectedPoint.y) + 180)
				if projectedPoint.x < 0 and projectedPoint.y < 0 then result = (atan (projectedPoint.x/projectedPoint.y) - 180)
				)
			)
	
		)--end case	
		if result == undefined then result = 0
		local delta_angle = result - norm_angle
		if delta_angle < -180 then windings += 1
		if delta_angle > 180 then windings -= 1
		target.angle = result + (windings * 360.0)
    )
)




plugin simpleManipulator DirectionManip
    name:"DirectionManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	
	
    -- This manipulator manipulates any modifier with an "Direction" property:
	--*Bend Direction
	--*Skew Direction
	
    on canManipulate target return
		(
		(try(if target.direction != undefined then true)catch(false)) and modifier_manipulators_enabled[2]
		)

    -- Create the manipulator gizmo.
    -- This is called initially and whenever the manipulator target changes
	
    on updateGizmos do
    (
        -- Clear the current gizmo cache
        this.clearGizmos()
	
		-- Create Gizmo 
		
		local size1 = length (node.max - node.min) /50
		if classof target == Skew then
		size2 = size1*30
		else
		size2 = size1*20
		
		giz2 = manip.MakeCircle [0,0,0] size2 24
		giz1 = manip.MakeSphere [size2*cos(-target.direction),size2*sin(-target.direction),0] size1 8
        giz3 = manip.makeGizmoShape()
        giz3.addPoint [size2*cos(-target.direction),size2*sin(-target.direction),0] 
        giz3.addPoint [0,0,0] 
 
        -- Add the Gizmos to the manipulator
        this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoShape giz2 gizmoDontHitTest greenColor redColor  
        this.addGizmoShape giz3 gizmoDontHitTest greenColor redColor  
		
		
        -- return the ToolTip string
        return node.name +":"+ target.name+ " Direction = " + target.direction as string 
    )

    -- mouseMove is called on every mouse move when dragging the manip
    -- It needs to convert the mouse position 'm' into a new value for the height
	
    on mouseMove m which do
    (
		
		local p1 = manip.makePlaneFromNormal z_axis [0, 0, 0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = p1.intersect viewRay &projectedPoint
        if (res) then 
			(
			tangens_value = projectedPoint.x/projectedPoint.y
			if projectedPoint.x >= 0 and projectedPoint.y > 0 then result = (atan (tangens_value)-90)
			if projectedPoint.x <= 0 and projectedPoint.y > 0 then result = (atan (tangens_value)-90)
			if projectedPoint.x >= 0 and projectedPoint.y < 0 then result = (atan (tangens_value) + 180-90)
			if projectedPoint.x <= 0 and projectedPoint.y < 0 then result = (atan (tangens_value) - 180-90)
			if result == undefined then result = 0
			target.direction = result
			)
    )
)




plugin simpleManipulator ModAmountManip
    name:"ModAmountManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local size = 3
	
    -- This manipulator manipulates any modifier with a "Amount" property:
	
    on canManipulate target return
		(
		(try(if target.amount != undefined then true)catch(false) ) and modifier_manipulators_enabled[3]
		)
		
    on updateGizmos do
    (
        this.clearGizmos()
		size = length (node.max - node.min) / 50		
		giz1 = manip.MakeSphere [0,0,target.amount+size*2] size 8
        giz2 = manip.makeGizmoShape()
        giz2.addPoint [0,0,target.amount+size] 
        giz2.addPoint [0,0,0] 
 
	
        this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoShape giz2 GizmoDontHitTest greenColor redColor  
        return node.name +":"+target.name+ " Amount = " + target.amount as string 
    )

	
    on mouseMove m which do
    (
        local plane1 = manip.makePlaneFromNormal x_axis [0, 0, 0]
        local plane2 = manip.makePlaneFromNormal y_axis [0, 0, 0]
        local projectedPoint = [0,0,0]

        viewRay = this.getLocalViewRay m
        local pl = plane1.mostOrthogonal viewRay plane2
        res = pl.intersect viewRay &projectedPoint
        if (res) then target.amount = (projectedPoint.z-size*2) 
    )
)

